@@ -42,7 +42,8 @@ int smb_read_error = 0;
 
 BOOL is_a_socket(int fd)
 {
-	int v,l;
+	int v;
+	socklen_t l;
 	l = sizeof(int);
 	return(getsockopt(fd, SOL_SOCKET, SO_TYPE, (char *)&v, &l) == 0);
 }
@@ -50,7 +51,7 @@ BOOL is_a_socket(int fd)
 enum SOCK_OPT_TYPES {OPT_BOOL,OPT_INT,OPT_ON};
 
 typedef struct smb_socket_option {
-	char *name;
+	const char *name;
 	int level;
 	int option;
 	int value;
@@ -99,7 +100,8 @@ smb_socket_option socket_options[] = {
 
 static void print_socket_options(int s)
 {
-	int value, vlen = 4;
+	int value;
+	socklen_t vlen = 4;
 	smb_socket_option *p = &socket_options[0];
 
 	for (; p->name != NULL; p++) {
@@ -115,7 +117,7 @@ static void print_socket_options(int s)
  Set user socket options.
 ****************************************************************************/
 
-void set_socket_options(int fd, char *options)
+void set_socket_options(int fd, const char *options)
 {
 	fstring tok;
 
@@ -604,7 +606,7 @@ BOOL receive_smb(int fd,char *buffer, unsigned int timeout)
 
 	len = read_smb_length_return_keepalive(fd,buffer,timeout);
 	if (len < 0) {
-		DEBUG(10,("receive_smb: length < 0!\n"));
+		DEBUG(10,("receive_smb: length < 0 !\n"));
 
 		/*
 		 * Correct fix. smb_read_error may have already been
@@ -931,7 +933,7 @@ static BOOL matchname(char *remotehost,struct in_addr  addr)
 	
 	/* Look up the host address in the address list we just got. */
 	for (i = 0; hp->h_addr_list[i]; i++) {
-		if (memcmp(hp->h_addr_list[i], (caddr_t) & addr, sizeof(addr)) == 0)
+		if (memcmp(hp->h_addr_list[i], (void *) & addr, sizeof(addr)) == 0)
 			return True;
 	}
 	
@@ -997,7 +999,7 @@ char *get_socket_addr(int fd)
 {
 	struct sockaddr sa;
 	struct sockaddr_in *sockin = (struct sockaddr_in *) (&sa);
-	int     length = sizeof(sa);
+	socklen_t length = sizeof(sa);
 	static fstring addr_buf;
 
 	fstrcpy(addr_buf,"0.0.0.0");
@@ -1019,102 +1021,97 @@ char *get_socket_addr(int fd)
 /*******************************************************************
  Create protected unix domain socket.
 
- some unixen cannot set permissions on a ux-dom-sock, so we
+ Some unixes cannot set permissions on a ux-dom-sock, so we
  have to make sure that the directory contains the protection
- permissions, instead.
+ permissions instead.
  ******************************************************************/
+
 int create_pipe_sock(const char *socket_dir,
-					const char *socket_name,
-					mode_t dir_perms)
+		     const char *socket_name,
+		     mode_t dir_perms)
 {
-        struct sockaddr_un sunaddr;
-        struct stat st;
-        int sock;
-        mode_t old_umask;
-        pstring path;
+#ifdef HAVE_UNIXSOCKET
+	struct sockaddr_un sunaddr;
+	struct stat st;
+	int sock;
+	mode_t old_umask;
+	pstring path;
         
-        /* Create the socket directory or reuse the existing one */
+	old_umask = umask(0);
         
-        if (lstat(socket_dir, &st) == -1) {
-                
-                if (errno == ENOENT) {
-                        
-                        /* Create directory */
-                        
-                        if (mkdir(socket_dir, dir_perms) == -1) {
-                                DEBUG(0, ("error creating socket directory "
-                                          "%s: %s\n", socket_dir, 
-                                          strerror(errno)));
-                                return -1;
-                        }
-                        
-                } else {
-                        
-                        DEBUG(0, ("lstat failed on socket directory %s: %s\n",
-                                  socket_dir, strerror(errno)));
-                        return -1;
-                }
-                
-        } else {
-                
-                /* Check ownership and permission on existing directory */
-                
-                if (!S_ISDIR(st.st_mode)) {
-                        DEBUG(0, ("socket directory %s isn't a directory\n",
-                                  socket_dir));
-                        return -1;
-                }
-                
-                if ((st.st_uid != sec_initial_uid()) || 
-                    ((st.st_mode & 0777) != dir_perms)) {
-                        DEBUG(0, ("invalid permissions on socket directory "
-                                  "%s\n", socket_dir));
-                        return -1;
-                }
-        }
+	/* Create the socket directory or reuse the existing one */
         
-        /* Create the socket file */
+	if (lstat(socket_dir, &st) == -1) {
+		if (errno == ENOENT) {
+			/* Create directory */
+			if (mkdir(socket_dir, dir_perms) == -1) {
+				DEBUG(0, ("error creating socket directory "
+					"%s: %s\n", socket_dir, 
+					strerror(errno)));
+				goto out_umask;
+			}
+		} else {
+			DEBUG(0, ("lstat failed on socket directory %s: %s\n",
+				socket_dir, strerror(errno)));
+			goto out_umask;
+		}
+	} else {
+		/* Check ownership and permission on existing directory */
+		if (!S_ISDIR(st.st_mode)) {
+			DEBUG(0, ("socket directory %s isn't a directory\n",
+				socket_dir));
+			goto out_umask;
+		}
+		if ((st.st_uid != sec_initial_uid()) || 
+				((st.st_mode & 0777) != dir_perms)) {
+			DEBUG(0, ("invalid permissions on socket directory "
+				"%s\n", socket_dir));
+			goto out_umask;
+		}
+	}
         
-        old_umask = umask(0);
+	/* Create the socket file */
         
-        sock = socket(AF_UNIX, SOCK_STREAM, 0);
+	sock = socket(AF_UNIX, SOCK_STREAM, 0);
         
-        if (sock == -1) {
-                perror("socket");
-		umask(old_umask);
-                return -1;
-        }
+	if (sock == -1) {
+		perror("socket");
+                goto out_umask;
+	}
         
-        snprintf(path, sizeof(path), "%s/%s", socket_dir, socket_name);
+	snprintf(path, sizeof(path), "%s/%s", socket_dir, socket_name);
         
-        unlink(path);
-        memset(&sunaddr, 0, sizeof(sunaddr));
-        sunaddr.sun_family = AF_UNIX;
-        safe_strcpy(sunaddr.sun_path, path, sizeof(sunaddr.sun_path)-1);
+	unlink(path);
+	memset(&sunaddr, 0, sizeof(sunaddr));
+	sunaddr.sun_family = AF_UNIX;
+	safe_strcpy(sunaddr.sun_path, path, sizeof(sunaddr.sun_path)-1);
         
-        if (bind(sock, (struct sockaddr *)&sunaddr, sizeof(sunaddr)) == -1) {
-                DEBUG(0, ("bind failed on pipe socket %s: %s\n",
-                          path,
-                          strerror(errno)));
-                close(sock);
-		umask(old_umask);
-                return -1;
-        }
+	if (bind(sock, (struct sockaddr *)&sunaddr, sizeof(sunaddr)) == -1) {
+		DEBUG(0, ("bind failed on pipe socket %s: %s\n", path,
+			strerror(errno)));
+		goto out_close;
+	}
         
-        if (listen(sock, 5) == -1) {
-                DEBUG(0, ("listen failed on pipe socket %s: %s\n",
-                          path,
-                          strerror(errno)));
-                close(sock);
-		umask(old_umask);
-                return -1;
-        }
+	if (listen(sock, 5) == -1) {
+		DEBUG(0, ("listen failed on pipe socket %s: %s\n", path,
+			strerror(errno)));
+		goto out_close;
+	}
         
-        umask(old_umask);
-        
-        /* Success! */
-        
-        return sock;
+	umask(old_umask);
+	return sock;
+
+out_close:
+	close(sock);
+
+out_umask:
+	umask(old_umask);
+	return -1;
+
+#else
+        DEBUG(0, ("create_pipe_sock: No Unix sockets on this system\n"));
+        return -1;
+#endif /* HAVE_UNIXSOCKET */
 }
 
 /*******************************************************************
