@@ -550,9 +550,9 @@ static char *get_static(char **buffer, int *buflen, int len)
    lib/util_str.c as I really don't want to have to link in any other
    objects if I can possibly avoid it. */
 
-BOOL next_token(char **ptr,char *buff,char *sep, size_t bufsize)
+BOOL next_token(const char **ptr,char *buff,const char *sep, size_t bufsize)
 {
-	char *s;
+	const char *s;
 	BOOL quoted;
 	size_t len=1;
 
@@ -592,7 +592,7 @@ BOOL next_token(char **ptr,char *buff,char *sep, size_t bufsize)
 
 static NSS_STATUS fill_pwent(struct passwd *result,
 				  struct winbindd_pw *pw,
-				  char **buffer, int *buflen)
+				  char **buffer, size_t *buflen)
 {
 	/* User name */
 
@@ -659,6 +659,17 @@ static NSS_STATUS fill_pwent(struct passwd *result,
 
 	strcpy(result->pw_shell, pw->pw_shell);
 
+	/* The struct passwd for Solaris has some extra fields which must
+	   be initialised or nscd crashes. */
+
+#if HAVE_PASSWD_PW_COMMENT
+	result->pw_comment = "";
+#endif
+
+#if HAVE_PASSWD_PW_AGE
+	result->pw_age = "";
+#endif
+
 	return NSS_STATUS_SUCCESS;
 }
 
@@ -667,7 +678,7 @@ static NSS_STATUS fill_pwent(struct passwd *result,
    Return NSS_STATUS_TRYAGAIN if we run out of memory. */
 
 static int fill_grent(struct group *result, struct winbindd_gr *gr,
-		      char *gr_mem, char **buffer, int *buflen)
+		      char *gr_mem, char **buffer, size_t *buflen)
 {
 	fstring name;
 	int i;
@@ -710,7 +721,7 @@ static int fill_grent(struct group *result, struct winbindd_gr *gr,
 	/* this next value is a pointer to a pointer so let's align it */
 
 	/* Calculate number of extra bytes needed to align on pointer size boundry */
-	if ((i = (int)*buffer % sizeof(char*)) != 0)
+	if ((i = (unsigned long)(*buffer) % sizeof(char*)) != 0)
 		i = sizeof(char*) - i;
 	
 	if ((tst = get_static(buffer, buflen, ((gr->num_gr_mem + 1) * 
@@ -734,7 +745,7 @@ static int fill_grent(struct group *result, struct winbindd_gr *gr,
 
 	i = 0;
 
-	while(next_token((char **)&gr_mem, name, ",", sizeof(fstring))) {
+	while(next_token((const char **)&gr_mem, name, ",", sizeof(fstring))) {
         
 		/* Allocate space for member */
         
@@ -856,7 +867,7 @@ _nss_winbind_getpwent_r(struct passwd *result, char *buffer,
 		}
 
 		ret = fill_pwent(result, &pw_cache[ndx_pw_cache],
-				 &buffer, &buflen);
+ &buffer, (int *)&buflen);
 		
 		/* Out of memory - try again */
 
@@ -906,7 +917,7 @@ _nss_winbind_getpwuid_r(uid_t uid, struct passwd *result, char *buffer,
 
 		if (ret == NSS_STATUS_SUCCESS) {
 			ret = fill_pwent(result, &response.data.pw, 
-					 &buffer, &buflen);
+					&buffer, (int *)&buflen);
 
 			if (ret == NSS_STATUS_TRYAGAIN) {
 				keep_response = True;
@@ -919,7 +930,7 @@ _nss_winbind_getpwuid_r(uid_t uid, struct passwd *result, char *buffer,
 
 		/* We've been called again */
 
-		ret = fill_pwent(result, &response.data.pw, &buffer, &buflen);
+		ret = fill_pwent(result, &response.data.pw, &buffer, (int *)&buflen);
 
 		if (ret == NSS_STATUS_TRYAGAIN) {
 			keep_response = True;
@@ -968,7 +979,7 @@ _nss_winbind_getpwnam_r(const char *name, struct passwd *result, char *buffer,
 
 		if (ret == NSS_STATUS_SUCCESS) {
 			ret = fill_pwent(result, &response.data.pw, &buffer,
-					 &buflen);
+					(int *)&buflen);
 
 			if (ret == NSS_STATUS_TRYAGAIN) {
 				keep_response = True;
@@ -981,7 +992,7 @@ _nss_winbind_getpwnam_r(const char *name, struct passwd *result, char *buffer,
 
 		/* We've been called again */
 
-		ret = fill_pwent(result, &response.data.pw, &buffer, &buflen);
+		ret = fill_pwent(result, &response.data.pw, &buffer, (int *)&buflen);
 
 		if (ret == NSS_STATUS_TRYAGAIN) {
 			keep_response = True;
@@ -1105,7 +1116,7 @@ _nss_winbind_getgrent_r(struct group *result,
 
 		ret = fill_grent(result, &gr_cache[ndx_gr_cache],
 				 ((char *)getgrent_response.extra_data)+mem_ofs,
-				 &buffer, &buflen);
+				&buffer, (int *)&buflen);
 		
 		/* Out of memory - try again */
 
@@ -1165,7 +1176,7 @@ _nss_winbind_getgrnam_r(const char *name,
 		if (ret == NSS_STATUS_SUCCESS) {
 			ret = fill_grent(result, &response.data.gr, 
 					 response.extra_data,
-					 &buffer, &buflen);
+					&buffer, (int *)&buflen);
 
 			if (ret == NSS_STATUS_TRYAGAIN) {
 				keep_response = True;
@@ -1179,7 +1190,7 @@ _nss_winbind_getgrnam_r(const char *name,
 		/* We've been called again */
 		
 		ret = fill_grent(result, &response.data.gr, 
-				 response.extra_data, &buffer, &buflen);
+				response.extra_data, &buffer, (int *)&buflen);
 		
 		if (ret == NSS_STATUS_TRYAGAIN) {
 			keep_response = True;
@@ -1228,7 +1239,7 @@ _nss_winbind_getgrgid_r(gid_t gid,
 
 			ret = fill_grent(result, &response.data.gr, 
 					 response.extra_data, 
-					 &buffer, &buflen);
+					&buffer, (int *)&buflen);
 
 			if (ret == NSS_STATUS_TRYAGAIN) {
 				keep_response = True;
@@ -1242,7 +1253,7 @@ _nss_winbind_getgrgid_r(gid_t gid,
 		/* We've been called again */
 
 		ret = fill_grent(result, &response.data.gr, 
-				 response.extra_data, &buffer, &buflen);
+				response.extra_data, &buffer, (int *)&buflen);
 
 		if (ret == NSS_STATUS_TRYAGAIN) {
 			keep_response = True;
