@@ -616,7 +616,7 @@ static void cache_mangled_name( char *mangled_name, char *raw_name )
  * ************************************************************************** **
  */
 
-static BOOL check_mangled_cache( char *s )
+static BOOL check_mangled_cache( char *s, size_t maxlen )
 {
   ubi_cacheEntryPtr FoundPtr;
   char             *ext_start = NULL;
@@ -654,7 +654,7 @@ static BOOL check_mangled_cache( char *s )
     if(saved_ext)
     {
       /* Replace the saved_ext as it was truncated. */
-      (void)pstrcat( s, saved_ext );
+      (void)safe_strcat( s, saved_ext, maxlen );
       SAFE_FREE(saved_ext);
     }
     return( False );
@@ -666,11 +666,11 @@ static BOOL check_mangled_cache( char *s )
 
   DEBUG( 3, ("Found %s on mangled stack ", s) );
 
-  (void)pstrcpy( s, found_name );
+  (void)safe_strcpy( s, found_name, maxlen );
   if( saved_ext )
   {
     /* Replace the saved_ext as it was truncated. */
-    (void)pstrcat( s, saved_ext );
+    (void)safe_strcat( s, saved_ext,maxlen );
     SAFE_FREE(saved_ext);
   }
 
